const Generator = require('yeoman-generator');
const path = require('path');
const { kebabCase } = require('lodash');
const _ = require('lodash');

module.exports = class AppGenerator extends Generator {
  constructor (args, opts) {
    super(args, opts);

    this.pkg = this.fs.readJSON(this.destinationPath('package.json'), {});

    const name = this.pkg.name || process.cwd().split(path.sep).pop()
    this.props = {
      name,
      description: this.pkg.description,
      src: this.pkg.directories && this.pkg.directories.lib
    };
    
  }

  prompting () {
    const dependencies = this.dependencies.concat(this.devDependencies)
    const prompts = [{
      name: 'name',
      message: 'Project name',
      when: !this.pkg.name,
      default: this.props.name,
      filter: kebabCase,
      validate (input) {

        const isSelfReferential = dependencies.some(dependency => {
        const separatorIndex = dependency.indexOf('@');
        const end = separatorIndex !== -1 ? separatorIndex : dependency.length;
        const dependencyName = dependency.substring(0, end);

        return dependencyName === input;
      });

        if (isSelfReferential) {
          return `Your project can not be named '${input}' because the '${input}' package will be installed as a project dependency.`;
        }

        return true;
      }
    }, {
      name: 'title',
      message: 'Title of your project',
      when: !this.options.title
    }, {
      name: 'description',
      message: 'Description',
      when: !this.pkg.description
    }];

    return this.prompt(prompts).then(props => {
      this.props = Object.assign(this.props, props);
    });
  }

  writing () {
    const props = this.props;
    const pkg = this.pkg = require(this.templatePath('package.json.js'))(this);
    this.fs.copy(this.templatePath('src'), this.destinationPath('src'));
    this.fs.copy(this.templatePath('_babelrc'), this.destinationPath('.babelrc'));
    this.fs.copy(this.templatePath('_eslintignore'), this.destinationPath('.eslintignore'));
    this.fs.copy(this.templatePath('_eslintrc'), this.destinationPath('.eslintrc'));
    this.fs.copy(this.templatePath('webpack.config.common.js'), this.destinationPath('webpack.config.common.js'));
    this.fs.copy(this.templatePath('webpack.config.dev.js'), this.destinationPath('webpack.config.dev.js'));
    this.fs.copy(this.templatePath('webpack.config.prod.js'), this.destinationPath('webpack.config.prod.js'));
    this.fs.copy(this.templatePath('public'), this.destinationPath('public'));
    this.fs.copyTpl(
      this.templatePath('public/index.html'),
      this.destinationPath('public/index.html'),
      {
        title: this.props.title || this.props.name
      }
    );
    this.fs.writeJSON(
      this.destinationPath('package.json'),
      pkg
    );
  }

}